@@ -1,11 +1,12 @@
 pub mod errors;
 pub mod form;
-pub mod range;
+mod range;
 pub mod request;
 pub mod response;
 
 pub use cookie;
 pub use errors::{HttpError, ReadError};
+pub use headers;
 pub use http::method::Method;
 pub use http::{header, method, uri, version, HeaderMap, HeaderValue, StatusCode};
 pub use hyper::body::HttpBody;
@@ -14,9 +15,7 @@ pub use range::HttpRange;
 pub use request::Request;
 pub use response::Response;
 
-pub use headers;
-
-pub fn guess_accept_mime(req: &Request, default_type: Option<Mime>) -> Mime {
+pub(crate) fn guess_accept_mime(req: &Request, default_type: Option<Mime>) -> Mime {
     let dmime: Mime = default_type.unwrap_or_else(|| "text/html".parse().unwrap());
     let accept = req.accept();
     accept.first().unwrap_or(&dmime).to_string().parse().unwrap_or(dmime)
