@@ -1,3 +1,5 @@
+//! Http response.
+
 use std::collections::HashMap;
 use std::fmt::{self, Debug};
 use std::net::SocketAddr;
@@ -69,29 +71,8 @@ impl Default for Request {
     }
 }
 
-impl Request {
-    /// Creates a new blank `Request`
-    pub fn new() -> Request {
-        Request {
-            uri: Uri::default(),
-            headers: HeaderMap::default(),
-            body: Some(Body::default()),
-            extensions: Extensions::default(),
-            method: Method::default(),
-            cookies: CookieJar::default(),
-            params: HashMap::new(),
-            queries: OnceCell::new(),
-            form_data: tokio::sync::OnceCell::new(),
-            payload: tokio::sync::OnceCell::new(),
-            version: Version::default(),
-            remote_addr: None,
-        }
-    }
-
-    /// Create a request from an hyper::Request.
-    ///
-    /// This constructor consumes the hyper::Request.
-    pub fn from_hyper(req: hyper::Request<Body>) -> Request {
+impl From<hyper::Request<Body>> for Request {
+    fn from(req: hyper::Request<Body>) -> Self {
         let (
             http::request::Parts {
                 method,
@@ -136,14 +117,34 @@ impl Request {
             remote_addr: None,
         }
     }
+}
+
+impl Request {
+    /// Creates a new blank `Request`
+    pub fn new() -> Request {
+        Request {
+            uri: Uri::default(),
+            headers: HeaderMap::default(),
+            body: Some(Body::default()),
+            extensions: Extensions::default(),
+            method: Method::default(),
+            cookies: CookieJar::default(),
+            params: HashMap::new(),
+            queries: OnceCell::new(),
+            form_data: tokio::sync::OnceCell::new(),
+            payload: tokio::sync::OnceCell::new(),
+            version: Version::default(),
+            remote_addr: None,
+        }
+    }
     /// Returns a reference to the associated URI.
     ///
     /// # Examples
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let request = Request::default();
-    /// assert_eq!(*request.uri(), *"/");
+    /// let req = Request::default();
+    /// assert_eq!(*req.uri(), *"/");
     /// ```
     #[inline]
     pub fn uri(&self) -> &Uri {
@@ -156,9 +157,9 @@ impl Request {
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let mut request: Request= Request::default();
-    /// *request.uri_mut() = "/hello".parse().unwrap();
-    /// assert_eq!(*request.uri(), *"/hello");
+    /// let mut req: Request= Request::default();
+    /// *req.uri_mut() = "/hello".parse().unwrap();
+    /// assert_eq!(*req.uri(), *"/hello");
     /// ```
     #[inline]
     pub fn uri_mut(&mut self) -> &mut Uri {
@@ -171,8 +172,8 @@ impl Request {
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let request = Request::default();
-    /// assert_eq!(*request.method(), Method::GET);
+    /// let req = Request::default();
+    /// assert_eq!(*req.method(), Method::GET);
     /// ```
     #[inline]
     pub fn method(&self) -> &Method {
@@ -205,9 +206,10 @@ impl Request {
         &mut self.version
     }
     #[inline]
-    pub fn set_remote_addr(&mut self, remote_addr: Option<SocketAddr>) {
+    pub(crate) fn set_remote_addr(&mut self, remote_addr: Option<SocketAddr>) {
         self.remote_addr = remote_addr;
     }
+    /// Get request remote address.
     #[inline]
     pub fn remote_addr(&self) -> Option<SocketAddr> {
         self.remote_addr
@@ -219,8 +221,8 @@ impl Request {
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let request = Request::default();
-    /// assert!(request.headers().is_empty());
+    /// let req = Request::default();
+    /// assert!(req.headers().is_empty());
     /// ```
     #[inline]
     pub fn headers(&self) -> &HeaderMap {
@@ -234,9 +236,9 @@ impl Request {
     /// ```
     /// # use salvo_core::http::*;
     /// # use salvo_core::http::header::*;
-    /// let mut request: Request = Request::default();
-    /// request.headers_mut().insert(HOST, HeaderValue::from_static("world"));
-    /// assert!(!request.headers().is_empty());
+    /// let mut req: Request = Request::default();
+    /// req.headers_mut().insert(HOST, HeaderValue::from_static("world"));
+    /// assert!(!req.headers().is_empty());
     /// ```
     #[inline]
     pub fn headers_mut(&mut self) -> &mut HeaderMap<HeaderValue> {
@@ -261,8 +263,8 @@ impl Request {
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let request = Request::default();
-    /// assert!(request.body().is_some());
+    /// let req = Request::default();
+    /// assert!(req.body().is_some());
     /// ```
     #[inline]
     pub fn body(&self) -> Option<&Body> {
@@ -286,8 +288,8 @@ impl Request {
     ///
     /// ```
     /// # use salvo_core::http::*;
-    /// let request = Request::default();
-    /// assert!(request.extensions().get::<i32>().is_none());
+    /// let req = Request::default();
+    /// assert!(req.extensions().get::<i32>().is_none());
     /// ```
     #[inline]
     pub fn extensions(&self) -> &Extensions {
@@ -301,15 +303,16 @@ impl Request {
     /// ```
     /// # use salvo_core::http::*;
     /// # use salvo_core::http::header::*;
-    /// let mut request: Request = Request::default();
-    /// request.extensions_mut().insert("hello");
-    /// assert_eq!(request.extensions().get(), Some(&"hello"));
+    /// let mut req = Request::default();
+    /// req.extensions_mut().insert("hello");
+    /// assert_eq!(req.extensions().get(), Some(&"hello"));
     /// ```
     #[inline]
     pub fn extensions_mut(&mut self) -> &mut Extensions {
         &mut self.extensions
     }
 
+    /// Get accept.
     pub fn accept(&self) -> Vec<Mime> {
         let mut list: Vec<Mime> = vec![];
         if let Some(accept) = self.headers.get("accept").and_then(|h| h.to_str().ok()) {
@@ -323,6 +326,7 @@ impl Request {
         list
     }
 
+    /// Get first accept.
     #[inline]
     pub fn frist_accept(&self) -> Option<Mime> {
         let mut accept = self.accept();
@@ -333,23 +337,26 @@ impl Request {
         }
     }
 
+    /// Get content type.
     #[inline]
     pub fn content_type(&self) -> Option<Mime> {
-        if let Some(ctype) = self.headers.get("content-type").and_then(|h| h.to_str().ok()) {
-            ctype.parse().ok()
-        } else {
-            None
-        }
+        self.headers
+            .get("content-type")
+            .and_then(|h| h.to_str().ok())
+            .and_then(|v| v.parse().ok())
     }
 
+    /// Get `CookieJar` reference.
     #[inline]
     pub fn cookies(&self) -> &CookieJar {
         &self.cookies
     }
+    /// Get `CookieJar` mutable reference.
     #[inline]
     pub fn cookies_mut(&mut self) -> &mut CookieJar {
         &mut self.cookies
     }
+    /// Get `Cookie` from cookies.
     #[inline]
     pub fn get_cookie<T>(&self, name: T) -> Option<&Cookie<'static>>
     where
@@ -357,15 +364,18 @@ impl Request {
     {
         self.cookies.get(name.as_ref())
     }
+    /// Get params reference.
     #[inline]
     pub fn params(&self) -> &HashMap<String, String> {
         &self.params
     }
+    /// Get params mutable reference.
     #[inline]
     pub fn params_mut(&mut self) -> &mut HashMap<String, String> {
         &mut self.params
     }
 
+    /// Get param value from params.
     #[inline]
     pub fn get_param<T>(&self, key: &str) -> Option<T>
     where
@@ -374,6 +384,7 @@ impl Request {
         self.params.get(key).and_then(|v| v.parse::<T>().ok())
     }
 
+    /// Get queries reference.
     pub fn queries(&self) -> &MultiMap<String, String> {
         self.queries.get_or_init(|| {
             form_urlencoded::parse(self.uri.query().unwrap_or_default().as_bytes())
@@ -381,6 +392,7 @@ impl Request {
                 .collect()
         })
     }
+    /// Get query value from queries.
     #[inline]
     pub fn get_query<F>(&self, key: &str) -> Option<F>
     where
@@ -389,6 +401,7 @@ impl Request {
         self.queries().get(key).and_then(|v| v.parse::<F>().ok())
     }
 
+    /// Get field data from form.
     #[inline]
     pub async fn get_form<F>(&mut self, key: &str) -> Option<F>
     where
@@ -401,10 +414,12 @@ impl Request {
             .and_then(|ps| ps.fields.get(key))
             .and_then(|v| v.parse::<F>().ok())
     }
+    /// Get `FilePart` reference from request.
     #[inline]
     pub async fn get_file(&mut self, key: &str) -> Option<&FilePart> {
         self.form_data().await.as_ref().ok().and_then(|ps| ps.files.get(key))
     }
+    /// Get `FilePart` lsit reference from request.
     #[inline]
     pub async fn get_files(&mut self, key: &str) -> Option<&Vec<FilePart>> {
         self.form_data()
@@ -413,6 +428,7 @@ impl Request {
             .ok()
             .and_then(|ps| ps.files.get_vec(key))
     }
+    /// Get value from form first if not found then get from query.
     #[inline]
     pub async fn get_form_or_query<F>(&mut self, key: &str) -> Option<F>
     where
@@ -420,6 +436,7 @@ impl Request {
     {
         self.get_form(key.as_ref()).await.or_else(|| self.get_query(key))
     }
+    /// Get value from query first if not found then get from form.
     #[inline]
     pub async fn get_query_or_form<F>(&mut self, key: &str) -> Option<F>
     where
@@ -427,58 +444,52 @@ impl Request {
     {
         self.get_query(key.as_ref()).or(self.get_form(key).await)
     }
+
+    /// Get request payload.
     pub async fn payload(&mut self) -> Result<&Vec<u8>, ReadError> {
-        let ctype = self
-            .headers()
-            .get(header::CONTENT_TYPE)
-            .and_then(|v| v.to_str().ok())
-            .unwrap_or("");
-        if ctype == "application/x-www-form-urlencoded" || ctype.starts_with("multipart/form-data") {
-            Err(ReadError::General(String::from("failed to read data1")))
-        } else if ctype.starts_with("application/json") || ctype.starts_with("text/") {
-            let body = self.body.take();
-            self.payload
-                .get_or_try_init(|| async {
-                    match body {
-                        Some(body) => read_body_bytes(body).await,
-                        None => Err(ReadError::General(String::from("failed to read data2"))),
-                    }
-                })
-                .await
-        } else {
-            Err(ReadError::General(String::from("failed to read data3")))
-        }
+        let body = self.body.take();
+        self.payload
+            .get_or_try_init(|| async {
+                match body {
+                    Some(body) => read_body_bytes(body).await,
+                    None => Err(ReadError::EmptyBody),
+                }
+            })
+            .await
     }
 
+    /// Get `FormData` reference from request.
     pub async fn form_data(&mut self) -> Result<&FormData, ReadError> {
         let ctype = self
             .headers()
             .get(header::CONTENT_TYPE)
             .and_then(|v| v.to_str().ok())
-            .unwrap_or("");
-        if ctype == "application/x-www-form-urlencoded" || ctype.starts_with("multipart/form-data") {
+            .unwrap_or_default();
+        if ctype == "application/x-www-form-urlencoded" || ctype.starts_with("multipart/") {
             let body = self.body.take();
             let headers = self.headers();
             self.form_data
                 .get_or_try_init(|| async {
                     match body {
                         Some(body) => form::read_form_data(headers, body).await,
-                        None => Err(ReadError::General("empty body".into())),
+                        None => Err(ReadError::EmptyBody),
                     }
                 })
                 .await
         } else {
-            Err(ReadError::General("failed to read form data".into()))
+            Err(ReadError::NotFormData)
         }
     }
 
+    /// Read body as text from request.
     #[inline]
     pub async fn read_text(&mut self) -> Result<&str, ReadError> {
-        match self.payload().await {
-            Ok(body) => Ok(std::str::from_utf8(body)?),
-            Err(_) => Err(ReadError::General("read text from body failed".into())),
-        }
+        self.payload()
+            .await
+            .and_then(|body| std::str::from_utf8(body).map_err(ReadError::Utf8))
     }
+
+    /// Read body as type `T` from request.
     #[inline]
     pub async fn read_from_text<T>(&mut self) -> Result<T, ReadError>
     where
@@ -486,32 +497,33 @@ impl Request {
     {
         self.read_text()
             .await
-            .and_then(|body| body.parse::<T>().map_err(|_| ReadError::Parsing(body.into())))
+            .and_then(|body| body.parse::<T>().map_err(|_| ReadError::ParseFromStr))
     }
+
+    /// Read body as type `T` from request.
     #[inline]
     pub async fn read_from_json<T>(&mut self) -> Result<T, ReadError>
     where
         T: DeserializeOwned,
     {
-        match self.payload().await {
-            Ok(body) => Ok(serde_json::from_slice::<T>(body)?),
-            Err(_) => Err(ReadError::General("read json from body failed".into())),
-        }
+        self.payload()
+            .await
+            .and_then(|body| serde_json::from_slice::<T>(body).map_err(ReadError::SerdeJson))
     }
+
+    /// Read body as type `T` from request.
     #[inline]
     pub async fn read_from_form<T>(&mut self) -> Result<T, ReadError>
     where
         T: DeserializeOwned,
     {
-        match self.form_data().await {
-            Ok(form_data) => {
-                let data = serde_json::to_value(&form_data.fields)?;
-                Ok(serde_json::from_value::<T>(data)?)
-            }
-            Err(_) => Err(ReadError::General("read data from form failed".into())),
-        }
+        self.form_data().await.and_then(|form_data| {
+            let data = serde_json::to_value(&form_data.fields)?;
+            Ok(serde_json::from_value::<T>(data)?)
+        })
     }
 
+    /// Read body as type `T` from request.
     #[inline]
     pub async fn read<T>(&mut self) -> Result<T, ReadError>
     where
@@ -521,28 +533,22 @@ impl Request {
             .headers()
             .get(header::CONTENT_TYPE)
             .and_then(|v| v.to_str().ok())
-            .unwrap_or("");
-        if ctype == "application/x-www-form-urlencoded" || ctype.starts_with("multipart/form-data") {
+            .unwrap_or_default();
+        if ctype == "application/x-www-form-urlencoded" || ctype.starts_with("multipart/") {
             self.read_from_form().await
         } else if ctype.starts_with("application/json") {
             self.read_from_json().await
         } else {
-            Err(ReadError::General(String::from(
-                "failed to read data or this type is not supported",
-            )))
+            Err(ReadError::InvalidContentType)
         }
     }
 }
 
-pub trait BodyReader: Send {}
-// pub(crate) async fn read_body_cursor<B: HttpBody>(body: B) -> Result<Cursor<Vec<u8>>, ReadError> {
-//     Ok(Cursor::new(read_body_bytes(body).await?))
-// }
 pub(crate) async fn read_body_bytes(body: Body) -> Result<Vec<u8>, ReadError> {
     hyper::body::to_bytes(body)
         .await
-        .map_err(|_| ReadError::General("read body bytes error".into()))
         .map(|d| d.to_vec())
+        .map_err(ReadError::Hyper)
 }
 
 #[cfg(test)]
@@ -554,13 +560,12 @@ mod tests {
 
     #[tokio::test]
     async fn test_read_text() {
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://127.0.0.1:7878/hello")
-                .header("content-type", "text/plain")
-                .body("hello".into())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://127.0.0.1:7878/hello")
+            .header("content-type", "text/plain")
+            .body("hello".into())
+            .unwrap()
+            .into();
         assert_eq!(req.read_from_text::<String>().await.unwrap(), "hello");
     }
     #[tokio::test]
@@ -569,13 +574,12 @@ mod tests {
         struct User {
             name: String,
         }
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://127.0.0.1:7878/hello")
-                .header("content-type", "application/json")
-                .body(r#"{"name": "jobs"}"#.into())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://127.0.0.1:7878/hello")
+            .header("content-type", "application/json")
+            .body(r#"{"name": "jobs"}"#.into())
+            .unwrap()
+            .into();
         assert_eq!(
             req.read_from_json::<User>().await.unwrap(),
             User { name: "jobs".into() }
@@ -583,56 +587,53 @@ mod tests {
     }
     #[tokio::test]
     async fn test_query() {
-        let mut request = Request::from_hyper(
-            hyper::Request::builder()
-                .method("GET")
-                .uri("http://127.0.0.1:7979/hello?q=rust")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
-        assert_eq!(request.queries().len(), 1);
-        assert_eq!(request.get_query::<String>("q").unwrap(), "rust");
-        assert_eq!(request.get_query_or_form::<String>("q").await.unwrap(), "rust");
+        let mut req: Request = hyper::Request::builder()
+            .method("GET")
+            .uri("http://127.0.0.1:7979/hello?q=rust")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
+        assert_eq!(req.queries().len(), 1);
+        assert_eq!(req.get_query::<String>("q").unwrap(), "rust");
+        assert_eq!(req.get_query_or_form::<String>("q").await.unwrap(), "rust");
     }
     #[tokio::test]
     async fn test_form() {
-        let mut request = Request::from_hyper(
-            hyper::Request::builder()
-                .method("POST")
-                .header("content-type", "application/x-www-form-urlencoded")
-                .uri("http://127.0.0.1:7979/hello?q=rust")
-                .body("lover=dog&money=sh*t&q=firefox".into())
-                .unwrap(),
-        );
-        assert_eq!(request.get_form::<String>("money").await.unwrap(), "sh*t");
-        assert_eq!(request.get_query_or_form::<String>("q").await.unwrap(), "rust");
-        assert_eq!(request.get_form_or_query::<String>("q").await.unwrap(), "firefox");
+        let mut req: Request = hyper::Request::builder()
+            .method("POST")
+            .header("content-type", "application/x-www-form-urlencoded")
+            .uri("http://127.0.0.1:7979/hello?q=rust")
+            .body("lover=dog&money=sh*t&q=firefox".into())
+            .unwrap()
+            .into();
+        assert_eq!(req.get_form::<String>("money").await.unwrap(), "sh*t");
+        assert_eq!(req.get_query_or_form::<String>("q").await.unwrap(), "rust");
+        assert_eq!(req.get_form_or_query::<String>("q").await.unwrap(), "firefox");
 
-        let mut request = Request::from_hyper(
-            hyper::Request::builder()
-                .method("POST")
-                .header(
-                    "content-type",
-                    "multipart/form-data; boundary=----WebKitFormBoundary0mkL0yrNNupCojyz",
-                )
-                .uri("http://127.0.0.1:7979/hello?q=rust")
-                .body(
-                    "------WebKitFormBoundary0mkL0yrNNupCojyz\r\n\
+        let mut req: Request = hyper::Request::builder()
+            .method("POST")
+            .header(
+                "content-type",
+                "multipart/form-data; boundary=----WebKitFormBoundary0mkL0yrNNupCojyz",
+            )
+            .uri("http://127.0.0.1:7979/hello?q=rust")
+            .body(
+                "------WebKitFormBoundary0mkL0yrNNupCojyz\r\n\
 Content-Disposition: form-data; name=\"money\"\r\n\r\nsh*t\r\n\
 ------WebKitFormBoundary0mkL0yrNNupCojyz\r\n\
 Content-Disposition: form-data; name=\"file1\"; filename=\"err.txt\"\r\n\
 Content-Type: text/plain\r\n\r\n\
 file content\r\n\
 ------WebKitFormBoundary0mkL0yrNNupCojyz--\r\n"
-                        .into(),
-                )
-                .unwrap(),
-        );
-        assert_eq!(request.get_form::<String>("money").await.unwrap(), "sh*t");
-        let file = request.get_file("file1").await.unwrap();
+                    .into(),
+            )
+            .unwrap()
+            .into();
+        assert_eq!(req.get_form::<String>("money").await.unwrap(), "sh*t");
+        let file = req.get_file("file1").await.unwrap();
         assert_eq!(file.file_name().unwrap(), "err.txt");
         assert_eq!(file.headers().get("content-type").unwrap(), "text/plain");
-        let files = request.get_files("file1").await.unwrap();
+        let files = req.get_files("file1").await.unwrap();
         assert_eq!(files[0].file_name().unwrap(), "err.txt");
     }
 }
