@@ -4,8 +4,10 @@ pub use filter::*;
 pub use router::{DetectMatched, Router};
 
 use std::collections::HashMap;
-pub type PathParams = HashMap<String, String>;
 
+#[doc(hidden)]
+pub type PathParams = HashMap<String, String>;
+#[doc(hidden)]
 #[derive(Debug, Eq, PartialEq)]
 pub struct PathState {
     pub(crate) url_path: String,
@@ -13,6 +15,7 @@ pub struct PathState {
     pub(crate) params: PathParams,
 }
 impl PathState {
+    /// Create new `PathState`.
     pub fn new(url_path: &str) -> Self {
         let url_path = url_path.trim_start_matches('/').trim_end_matches('/');
         PathState {
