@@ -6,6 +6,7 @@ use super::{Filter, FnFilter, PathFilter, PathState};
 use crate::http::Request;
 use crate::Handler;
 
+/// Router struct is used for route request to different handlers.
 pub struct Router {
     pub(crate) routers: Vec<Router>,
     pub(crate) filters: Vec<Box<dyn Filter>>,
@@ -13,10 +14,11 @@ pub struct Router {
     pub(crate) befores: Vec<Arc<dyn Handler>>,
     pub(crate) afters: Vec<Arc<dyn Handler>>,
 }
+#[doc(hidden)]
 pub struct DetectMatched {
-    pub handler: Arc<dyn Handler>,
-    pub befores: Vec<Arc<dyn Handler>>,
-    pub afters: Vec<Arc<dyn Handler>>,
+    pub handler: (u64, Arc<dyn Handler>),
+    pub befores: Vec<(u64, Arc<dyn Handler>)>,
+    pub afters: Vec<(u64, Arc<dyn Handler>)>,
 }
 
 impl Default for Router {
@@ -82,7 +84,7 @@ impl Router {
     }
 
     /// Detect current router is matched for current request.
-    pub fn detect(&self, req: &mut Request, path_state: &mut PathState) -> Option<DetectMatched> {
+    pub fn detect(&self, req: &mut Request, path_state: &mut PathState, depth: u64) -> Option<DetectMatched> {
         for filter in &self.filters {
             if !filter.filter(req, path_state) {
                 return None;
@@ -91,10 +93,14 @@ impl Router {
         if !self.routers.is_empty() {
             let original_cursor = path_state.cursor;
             for child in &self.routers {
-                if let Some(dm) = child.detect(req, path_state) {
+                if let Some(dm) = child.detect(req, path_state, depth + 1) {
+                    let self_befores: Vec<(u64, Arc<dyn Handler>)> =
+                        self.befores.iter().map(|before| (depth, Arc::clone(before))).collect();
+                    let self_afters: Vec<(u64, Arc<dyn Handler>)> =
+                        self.afters.iter().map(|after| (depth, Arc::clone(after))).collect();
                     return Some(DetectMatched {
-                        befores: [&self.befores[..], &dm.befores[..]].concat(),
-                        afters: [&dm.afters[..], &self.afters[..]].concat(),
+                        befores: [&self_befores[..], &dm.befores[..]].concat(),
+                        afters: [&dm.afters[..], &self_afters[..]].concat(),
                         handler: dm.handler.clone(),
                     });
                 } else {
@@ -105,9 +111,19 @@ impl Router {
         if let Some(handler) = self.handler.clone() {
             if path_state.ended() {
                 return Some(DetectMatched {
-                    befores: self.befores.clone(),
-                    afters: self.afters.clone(),
-                    handler: handler.clone(),
+                    befores: self
+                        .befores
+                        .clone()
+                        .iter()
+                        .map(|before| (depth, Arc::clone(before)))
+                        .collect(),
+                    afters: self
+                        .afters
+                        .clone()
+                        .iter()
+                        .map(|after| (depth, Arc::clone(after)))
+                        .collect(),
+                    handler: (depth, handler.clone()),
                 });
             }
         }
@@ -128,18 +144,6 @@ impl Router {
         self
     }
 
-    #[deprecated(since = "0.10.4", note = "Please use then function instead")]
-    #[inline]
-    pub fn push_when<F>(mut self, func: F) -> Self
-    where
-        F: Fn(&Router) -> Option<Router>,
-    {
-        if let Some(router) = func(&self) {
-            self.routers.push(router);
-        }
-        self
-    }
-
     /// Add a handler as middleware, it will run before the handler in current router or it's descendants
     /// handle the request.
     #[inline]
@@ -191,7 +195,6 @@ impl Router {
     }
 
     /// Add a filter for current router.
-    ///
     #[inline]
     pub fn filter(mut self, filter: impl Filter + Sized) -> Self {
         self.filters.push(Box::new(filter));
@@ -217,7 +220,6 @@ impl Router {
 
     /// When you want write router chain, this function will be useful,
     /// You can write your custom logic in FnOnce.
-    ///
     #[inline]
     pub fn then<F>(self, func: F) -> Self
     where
@@ -274,8 +276,8 @@ mod tests {
     use super::{PathState, Router};
     use crate::fn_handler;
     use crate::{Request, Response};
-
     use async_trait::async_trait;
+
     #[fn_handler]
     async fn fake_handler(_res: &mut Response) {}
     #[test]
@@ -284,14 +286,13 @@ mod tests {
             Router::with_path("users")
                 .push(Router::with_path("<id>").push(Router::with_path("emails").get(fake_handler))),
         );
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/emails")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/emails")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
@@ -302,14 +303,13 @@ mod tests {
                 Router::with_path("users")
                     .push(Router::with_path("<id>").push(Router::with_path("emails").get(fake_handler))),
             );
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/emails")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/emails")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
@@ -320,24 +320,22 @@ mod tests {
                     .push(Router::new().push(Router::with_path("facebook/insights/<**rest>").handle(fake_handler))),
             ),
         );
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights/23")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights/23")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         // assert_eq!(format!("{:?}", path_state), "");
         assert!(matched.is_some());
     }
@@ -349,25 +347,23 @@ mod tests {
                     .push(Router::new().push(Router::with_path("facebook/insights/<*rest>").handle(fake_handler))),
             ),
         );
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         // assert_eq!(format!("{:?}", path_state), "");
         assert!(matched.is_none());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights/23")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights/23")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
@@ -378,24 +374,22 @@ mod tests {
                     Router::with_path("facebook/insights").push(Router::with_path("<**rest>").handle(fake_handler)),
                 ),
             )));
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights/23")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights/23")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
         assert_eq!(path_state.params["id"], "12");
     }
@@ -407,24 +401,22 @@ mod tests {
                     Router::with_path("facebook/insights").push(Router::new().path("<*rest>").handle(fake_handler)),
                 ),
             )));
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_none());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/facebook/insights/23")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/facebook/insights/23")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
@@ -435,71 +427,65 @@ mod tests {
                     Router::with_path("facebook/insights").push(Router::with_path("<*rest>").handle(fake_handler)),
                 ),
             )));
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/%E7%94%A8%E6%88%B7/12/facebook/insights")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/%E7%94%A8%E6%88%B7/12/facebook/insights")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_none());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/%E7%94%A8%E6%88%B7/12/facebook/insights/23")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/%E7%94%A8%E6%88%B7/12/facebook/insights/23")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
     fn test_router_detect9() {
         let router =
             Router::new().push(Router::with_path("users/<*sub:/(images|css)/>/<filename>").handle(fake_handler));
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/m.jpg")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/m.jpg")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_none());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/css/m.jpg")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/css/m.jpg")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
     #[test]
     fn test_router_detect10() {
         let router = Router::new().push(Router::with_path(r"users/<*sub:/(images|css)/.+/>").handle(fake_handler));
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/12/m.jpg")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/12/m.jpg")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_none());
 
-        let mut req = Request::from_hyper(
-            hyper::Request::builder()
-                .uri("http://local.host/users/css/abc/m.jpg")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
+        let mut req: Request = hyper::Request::builder()
+            .uri("http://local.host/users/css/abc/m.jpg")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
         let mut path_state = PathState::new(req.uri().path());
-        let matched = router.detect(&mut req, &mut path_state);
+        let matched = router.detect(&mut req, &mut path_state, 0);
         assert!(matched.is_some());
     }
 }
