@@ -7,8 +7,10 @@ use crate::http::header::HeaderValue;
 use crate::http::{Request, Response};
 use crate::Depot;
 
+/// Writer is used to write data to response.
 #[async_trait]
 pub trait Writer {
+    /// Write data to ```Respone```.
     #[must_use = "write future must be used"]
     async fn write(mut self, req: &mut Request, depot: &mut Depot, res: &mut Response);
 }
@@ -54,6 +56,7 @@ impl Writer for String {
     }
 }
 
+/// Write text content to response as text content. It will set ```content-type``` to ```text/plain; charset=utf-8```.
 pub struct PlainText<T>(T);
 #[async_trait]
 impl<T> Writer for PlainText<T>
@@ -68,6 +71,7 @@ where
     }
 }
 
+/// Write text content to response as json content. It will set ```content-type``` to ```application/json; charset=utf-8```.
 pub struct JsonText<T>(T);
 #[async_trait]
 impl<T> Writer for JsonText<T>
@@ -90,6 +94,7 @@ where
     }
 }
 
+/// Write text content to response as html content. It will set ```content-type``` to ```text/html; charset=utf-8```.
 pub struct HtmlText<T>(T);
 #[async_trait]
 impl<T> Writer for HtmlText<T>
@@ -104,6 +109,7 @@ where
     }
 }
 
+/// Write serializable content to response as json content. It will set ```content-type``` to ```application/json; charset=utf-8```.
 pub struct JsonContent<T>(T);
 #[async_trait]
 impl<T> Writer for JsonContent<T>
@@ -130,14 +136,13 @@ mod tests {
     use super::*;
 
     async fn access(service: &Service) -> Response {
-        let request = Request::from_hyper(
-            hyper::Request::builder()
-                .method("GET")
-                .uri("http://127.0.0.1:7979/test")
-                .body(hyper::Body::empty())
-                .unwrap(),
-        );
-        service.handle(request).await
+        let req: Request = hyper::Request::builder()
+            .method("GET")
+            .uri("http://127.0.0.1:7979/test")
+            .body(hyper::Body::empty())
+            .unwrap()
+            .into();
+        service.handle(req).await
     }
 
     #[tokio::test]
@@ -150,12 +155,9 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), "hello");
-        assert_eq!(
-            response.headers().get("content-type").unwrap(),
-            "text/plain; charset=utf-8"
-        );
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), "hello");
+        assert_eq!(res.headers().get("content-type").unwrap(), "text/plain; charset=utf-8");
     }
 
     #[tokio::test]
@@ -168,12 +170,9 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), "hello");
-        assert_eq!(
-            response.headers().get("content-type").unwrap(),
-            "text/plain; charset=utf-8"
-        );
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), "hello");
+        assert_eq!(res.headers().get("content-type").unwrap(), "text/plain; charset=utf-8");
     }
 
     #[tokio::test]
@@ -186,12 +185,9 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), "hello");
-        assert_eq!(
-            response.headers().get("content-type").unwrap(),
-            "text/plain; charset=utf-8"
-        );
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), "hello");
+        assert_eq!(res.headers().get("content-type").unwrap(), "text/plain; charset=utf-8");
     }
 
     #[tokio::test]
@@ -204,10 +200,10 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), r#"{"hello": "world"}"#);
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), r#"{"hello": "world"}"#);
         assert_eq!(
-            response.headers().get("content-type").unwrap(),
+            res.headers().get("content-type").unwrap(),
             "application/json; charset=utf-8"
         );
     }
@@ -240,10 +236,10 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), r#"{"name":"jobs"}"#);
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), r#"{"name":"jobs"}"#);
         assert_eq!(
-            response.headers().get("content-type").unwrap(),
+            res.headers().get("content-type").unwrap(),
             "application/json; charset=utf-8"
         );
     }
@@ -258,11 +254,8 @@ mod tests {
         let router = Router::new().push(Router::with_path("test").get(test));
         let service = Service::new(router);
 
-        let mut response = access(&service).await;
-        assert_eq!(response.take_text().await.unwrap(), "<html><body>hello</body></html>");
-        assert_eq!(
-            response.headers().get("content-type").unwrap(),
-            "text/html; charset=utf-8"
-        );
+        let mut res = access(&service).await;
+        assert_eq!(res.take_text().await.unwrap(), "<html><body>hello</body></html>");
+        assert_eq!(res.headers().get("content-type").unwrap(), "text/html; charset=utf-8");
     }
 }
